# frozen_string_literal: true

require 'pathname'

module QA
  RSpec.describe 'Secure', :runner do
    describe 'Security Reports in a Merge Request' do
      let(:sast_vuln_count) { 5 }
      let(:dependency_scan_vuln_count) { 4 }
      let(:container_scan_vuln_count) { 8 }
      let(:vuln_name) { "Regular Expression Denial of Service in debug" }
      let(:remediable_vuln_name) { "Authentication bypass via incorrect DOM traversal and canonicalization in saml2-js" }

      after(:all) do
        @runner.remove_via_api!
      end

      before(:all) do
        @executor = "qa-runner-#{Time.now.to_i}"

        Flow::Login.sign_in

        @project = Resource::Project.fabricate_via_api! do |p|
          p.name = Runtime::Env.auto_devops_project_name || 'project-with-secure'
          p.description = 'Project with Secure'
          p.auto_devops_enabled = false
          p.initialize_with_readme = true
        end

        @runner = Resource::Runner.fabricate! do |runner|
          runner.project = @project
          runner.name = @executor
          runner.tags = %w[qa test]
        end

        @source = Resource::Repository::ProjectPush.fabricate! do |push|
          push.project = @project
          push.directory = Pathname
                               .new(__dir__)
                               .join('../../../../../ee/fixtures/secure_premade_reports')
          push.commit_message = 'Create Secure compatible application to serve premade reports'
          push.branch_name = 'secure-mr'
        end

        @merge_request = Resource::MergeRequest.fabricate_via_api! do |mr|
          mr.project = @project
          mr.source_branch = 'secure-mr'
          mr.target_branch = @project.default_branch
          mr.source = @source
          mr.target = @project.default_branch
          mr.target_new_branch = false
        end

        @project.visit!
        Flow::Pipeline.wait_for_latest_pipeline(pipeline_condition: 'succeeded')
      end

      before do
        Flow::Login.sign_in_unless_signed_in
        @merge_request.visit!
      end

      it 'can dismiss a vulnerability with a reason', testcase: 'https://gitlab.com/gitlab-org/quality/testcases/-/issues/1691' do
        dismiss_reason = "Vulnerability not applicable"

        Page::MergeRequest::Show.perform do |merge_request|
          expect(merge_request).to have_vulnerability_report
          merge_request.dismiss_vulnerability_with_reason(vuln_name, dismiss_reason)
          merge_request.click_vulnerability(vuln_name)

          expect(merge_request).to have_opened_dismissed_vulnerability(dismiss_reason)
        end
      end

      it 'can create an issue from a vulnerability', testcase: 'https://gitlab.com/gitlab-org/quality/testcases/-/issues/548' do
        Page::MergeRequest::Show.perform do |merge_request|
          expect(merge_request).to have_vulnerability_report
          merge_request.create_vulnerability_issue(vuln_name)
        end

        Page::Project::Issue::Show.perform do |issue|
          expect(issue).to have_title("Investigate vulnerability: #{vuln_name}")
        end
      end

      it 'can create an auto-remediation MR', testcase: 'https://gitlab.com/gitlab-org/quality/testcases/-/issues/550' do
        Page::MergeRequest::Show.perform do |merge_request|
          expect(merge_request).to have_vulnerability_report
          merge_request.resolve_vulnerability_with_mr remediable_vuln_name

          # Context changes as resolve method creates new MR
          expect(merge_request).to have_title remediable_vuln_name
        end
      end
    end
  end
end
